/*	mod_mysql.c
   **
   ** Copyright (c) 1995-96  Hughes Technologies Pty Ltd.
   **
   ** Permission to use, copy, and distribute for non-commercial purposes,
   ** is hereby granted without fee, providing that the above copyright
   ** notice appear in all copies and that both the copyright notice and this
   ** permission notice appear in supporting documentation.
   **
   ** The software may be modified for your own purposes, but modified versions
   ** may not be distributed.
   **
   ** This software is provided "as is" without any expressed or implied warranty.
   **
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <time.h>
#include <dlfcn.h>
#include <lite.h>
#include <mysql/mysql.h>

#include "mod_mysql.h"


extern sym_t *externReturn;


void doMysqlConnect(params)
plist_t *params;
{
    char *host = NULL, *user = NULL, *passwd = NULL;
    MYSQL *sock = NULL;

    if (params) {
	if (params->sym->type != TYPE_CHAR) {
	    setError("Invalid parameter");
	    externReturn = createIntSymbol(-1);
	    return;
	}
	host = params->sym->val[0] ? params->sym->val : NULL;
	user = params->next->sym->val[0] ? params->next->sym->val : NULL;
	passwd = params->next->next->sym->val[0] ? params->next->next->sym->val : NULL;
    }
    sock = malloc(sizeof(MYSQL));
    if (!(sock = mysql_connect(sock, host, user, passwd))) {
	setError(mysql_error(sock));
	externReturn = createIntSymbol(-1);
    } else {
	externReturn = createIntSymbol((int)sock);
    }
}


#define DECLARE_SOCK  MYSQL *sock = (MYSQL *) *(int *)params->sym->val
#define DECLARE_RES  MYSQL_RES *res = (MYSQL_RES *) *(int *)params->sym->val


void doMysqlClose(params)
plist_t *params;
{
    DECLARE_SOCK;

    mysql_close(sock);
    externReturn = createIntSymbol(0);
}


void doMysqlSelectDB(params)
plist_t *params;
{
    char *dbName;
    DECLARE_SOCK;

    dbName = params->next->sym->val;
    if (mysql_select_db(sock, dbName) < 0) {
	setError(mysql_error(sock));
	externReturn = createIntSymbol(-1);
    } else {
	externReturn = createIntSymbol(0);
    }
}


void doMysqlQuery(params)
plist_t *params;
{
    char *query;
    int res;
    DECLARE_SOCK;

    query = params->next->sym->val;
    res = mysql_query(sock, query);
    if (res) {
	setError(mysql_error(sock));
    }
    externReturn = createIntSymbol(res);
}


void doMysqlStoreResult(params)
plist_t *params;
{
    MYSQL_RES *res;
    DECLARE_SOCK;

    res = mysql_store_result(sock);
    if (!res) {
	setError(mysql_error(sock));
    }
    externReturn = createIntSymbol((int)res);
}


void doMysqlFreeResult(params)
plist_t *params;
{
    DECLARE_RES;

    mysql_free_result(res);
}


void doMysqlFetchRow(params)
plist_t *params;
{
    int numFields, index;
    sym_t *array, *sym;
    MYSQL_ROW row;
    DECLARE_RES;

    row = mysql_fetch_row(res);
    if (!row) {
	return;
    }
    array = createArray();
    numFields = mysql_num_fields(res);
    index = 0;
    while (index < numFields) {
	if (row[index])
	    sym = createCharSymbol(row[index]);
	else
	    sym = createCharSymbol("");
	symSetArrayElement(array, index, sym);
	index++;
    }
    externReturn = array;
}


void doMysqlDataSeek(params)
plist_t *params;
{
    int pos;
    DECLARE_RES;

    pos = (int) *(int *) params->next->sym->val;
    mysql_data_seek(res, pos);
}


void doMysqlNumRows(params)
plist_t *params;
{
    int numRows;
    DECLARE_RES;

    numRows = mysql_num_rows(res);
    externReturn = createIntSymbol(numRows);
}


void doMysqlFetchField(params)
plist_t *params;
{
    int index;
    sym_t *array, *sym;
    MYSQL_FIELD *field;
    DECLARE_RES;

    field = mysql_fetch_field(res);
    if (!field) {
	return;
    }
    array = createArray();
    index = 0;
    sym = createCharSymbol(field->name);
    symSetArrayElement(array, index++, sym);
    sym = createCharSymbol(field->table);
    symSetArrayElement(array, index++, sym);
    sym = createIntSymbol(field->type);
    symSetArrayElement(array, index++, sym);
    sym = createIntSymbol(field->length);
    symSetArrayElement(array, index++, sym);
    sym = createIntSymbol(field->flags);
    symSetArrayElement(array, index++, sym);
    externReturn = array;
}


void doMysqlFieldSeek(params)
plist_t *params;
{
    int pos;
    DECLARE_RES;

    pos = (int) *(int *) params->next->sym->val;
    mysql_field_seek(res, pos);
}


void doMysqlNumFields(params)
plist_t *params;
{
    int numFields;
    DECLARE_RES;

    numFields = mysql_num_fields(res);
    externReturn = createIntSymbol(numFields);
}


void doMysqlListDBs(params)
plist_t *params;
{
    MYSQL_RES *res;
    MYSQL_ROW cur;
    sym_t *array, *sym;
    int index;
    DECLARE_SOCK;

    res = mysql_list_dbs(sock, NULL);
    if (!res) {
	return;
    }
    index = 0;
    array = createArray();
    cur = mysql_fetch_row(res);
    while (cur) {
	sym = createCharSymbol(cur[0]);
	symSetArrayElement(array, index++, sym);
	cur = mysql_fetch_row(res);
    }
    mysql_free_result(res);
    externReturn = array;
}


void doMysqlListTables(params)
plist_t *params;
{
    MYSQL_RES *res;
    MYSQL_ROW cur;
    sym_t *array, *sym;
    int index;
    DECLARE_SOCK;

    res = mysql_list_tables(sock, NULL);
    if (!res) {
	return;
    }
    index = 0;
    array = createArray();
    cur = mysql_fetch_row(res);
    while (cur) {
	sym = createCharSymbol(cur[0]);
	symSetArrayElement(array, index++, sym);
	cur = mysql_fetch_row(res);
    }
    mysql_free_result(res);
    externReturn = array;
}


static MYSQL_RES *fields = NULL;

void doMysqlInitFieldList(params)
plist_t *params;
{
    char *table;
    DECLARE_SOCK;

    table = params->next->sym->val;
    if (fields) {
	mysql_free_result(fields);
    }
    fields = mysql_list_fields(sock, table, NULL);
    if (!fields) {
	externReturn = createIntSymbol(-1);
    } else {
	externReturn = createIntSymbol(0);
    }
}


void doMysqlListField(params)
plist_t *params;
{
    sym_t *array, *sym;
    MYSQL_FIELD *field;
    int index;

    if (!fields) {
	return;
    }
    field = mysql_fetch_field(fields);
    if (!field) {
	return;
    }
    array = createArray();
    index = 0;
    sym = createCharSymbol(field->name);
    symSetArrayElement(array, index++, sym);
    sym = createCharSymbol(field->table);
    symSetArrayElement(array, index++, sym);
    sym = createIntSymbol(field->type);
    symSetArrayElement(array, index++, sym);
    sym = createIntSymbol(field->length);
    symSetArrayElement(array, index++, sym);
    sym = createIntSymbol(field->flags);
    symSetArrayElement(array, index++, sym);
    externReturn = array;
}


void doMysqlEncode(params)
plist_t *params;
{
    char *str, *new;
    int len;

    str = params->sym->val;
    len = strlen(str);
    new = (char *) malloc(2 * len + 1);
    mysql_escape_string(new, str, len);
    externReturn = createCharSymbol(new);
    free(new);
}


int init_mod(modPtr)
void *modPtr;
{
    if (dlopen("/usr/local/lib/mysql/libmysqlclient.so", RTLD_LAZY | RTLD_GLOBAL) == NULL) {
	fprintf(stderr, "error loading /usr/local/lib/mysql/libmysqlclient.so: %s\n", dlerror());
	return -1;
    }
    if (modLoadModuleFunctions(modPtr, mysql_efuncts) < 0) {
	return -1;
    } else {
	return 0;
    }
}

